@@ -221,7 +221,7 @@ class Kbs:
             if cluster_client.IsClusterClass(cluster["clusterName"]):
                 cluster_client.cc_update(cluster["clusterName"])
             else:
-                cluster_client.update()
+                cluster_client.update(cluster["clusterName"])
         except Exception as e:
             logger.critical("failed to update mgmt cluster configmap/crs with errors: %s" % repr(e))
             traceback.print_exc()
@@ -285,6 +285,8 @@ class ClusterClient:
         return self.run_cmd(get_cmd)
 
 class TkgClusterClient(ClusterClient):
+    FQDN_ANNO = "telco.vmware.com/airgap-fqdn"
+    CA_ANNO = "telco.vmware.com/airgap-ca-cert"
     PATCH_KAPP_CM_CMD = "patch cm -n tkg-system kapp-controller-config -p"
     GET_KAPP_CM_CMD = "cm -n tkg-system kapp-controller-config"
     RESTART_KAPP_DEPLOYMENT_CMD = "rollout restart deployment -n tkg-system kapp-controller"
@@ -355,6 +357,47 @@ class TkgClusterClient(ClusterClient):
                 return
         logger.error("configmap kapp-controller-config/tkg-system: out of date")
 
+    def update_cluster_anno_ca(self, cluster_name, ns):
+        base64_ca = self.repo.get_base64_ca()
+        patch_cmd = "annotate --overwrite tkc %s -n %s %s=%s" % (cluster_name, ns, self.CA_ANNO, base64_ca)
+        self.run_cmd(patch_cmd)
+        logger.info("update cluster %s tkc airgap ca annotation successfully" % cluster_name)
+
+    def update_cluster_spec_ca(self, cluster_name, ns):
+        base64_ca = self.repo.get_base64_ca()
+        patch_json = "{\"spec\":{\"airgap\":{\"caCert\":\"%s\"}}}" % base64_ca
+        patch_cmd = "patch tkc %s -n %s --type=merge -p \'%s\'" % (cluster_name, ns, patch_json)
+        self.run_cmd(patch_cmd)
+        logger.info("update cluster %s tkc spec airgap cacert successfully" % cluster_name)
+    
+    def verify_tkc(self, clusterName):
+        cmd = "tkc -n %s %s" % (clusterName, clusterName)
+        tkc = self.get_dict(cmd)
+        fqdn = self.repo.get_fqdn()
+        base64_ca = self.repo.get_base64_ca()
+        check = False
+        if 'airgap' in tkc['spec'] and tkc['spec']['airgap']['fqdn'] == fqdn:
+            check = True
+            if ('caCert' not in tkc['spec']['airgap'] or
+                tkc['spec']['airgap']['caCert'] != base64_ca):
+                logger.error("tcakubenetescluster cr[%s/%s] spec airgap ca cert: out of date" % (clusterName, clusterName))
+            else:
+                logger.info("tcakubenetescluster cr[%s/%s] spec airgap ca cert: up to date" % (clusterName, clusterName))
+        
+        if ('annotations' in tkc['metadata'] and
+              self.FQDN_ANNO in tkc['metadata']['annotations'] and
+              tkc['metadata']['annotations'][self.FQDN_ANNO] == fqdn):
+            check = True
+            if (self.CA_ANNO not in tkc['metadata']['annotations'] or
+                tkc['metadata']['annotations'][self.CA_ANNO] != base64_ca):
+                logger.error("tcakubenetescluster cr[%s/%s] airgap annotation ca cert: out of date" % (clusterName, clusterName))
+            else:
+                logger.info("tcakubenetescluster cr[%s/%s] airgap annotation ca cert: up to date" % (clusterName, clusterName))
+
+        if not check:
+            logger.debug("skip tcakubernetescluster cr[%s/%s]: non target airgap repo"
+                        % (clusterName, clusterName))
+
 class MgmtClusterClient(TkgClusterClient):
     PATCH_TKR_CM_CMD_TKR_SYSTEM = "patch cm -n tkr-system tkr-controller-config -p"
     GET_TKR_CM_CMD_TKR_SYSTEM = "cm -n tkr-system tkr-controller-config"
@@ -773,6 +816,7 @@ class MgmtClusterClient(TkgClusterClient):
                          secret_name, ns))
             
     def cc_update(self, clusterName):
+        self.update_tkc(clusterName)
         self.update_kapp_controller_cm()
         self.update_tkg_pkg_secret()
         self.update_tkr_controller_secrets()
@@ -803,6 +847,7 @@ class MgmtClusterClient(TkgClusterClient):
 
     def cc_verify(self, clusterName):
         verify_nodeconfig(self.repo, self)
+        self.verify_tkc(clusterName)
         self.verify_kapp_controller_cm()
         self.verify_tkg_pkg_secret()
         self.verify_tkr_controller_secrets()
@@ -825,7 +870,7 @@ class MgmtClusterClient(TkgClusterClient):
         
         self.verify_machine()
 
-    def update(self):
+    def update(self, clusterName):
         fqdn = self.repo.get_fqdn()
         ssl_cert_file = "/etc/ssl/certs/%s.pem" % fqdn
         self.update_kapp_controller_cm()
@@ -835,9 +880,11 @@ class MgmtClusterClient(TkgClusterClient):
         self.update_kcp("tkg-system")
         self.update_kcts("tkg-system")
         self.update_nodeconfig(ssl_cert_file)
+        self.update_tkc(clusterName)
 
-    def verify(self):
+    def verify(self, clusterName):
         verify_nodeconfig(self.repo, self)
+        self.verify_tkc(clusterName)
         self.verify_kapp_controller_cm()
         self.verify_tkg_pkg_secret()
         self.verify_tkr_controller_secrets()
@@ -892,6 +939,21 @@ class MgmtClusterClient(TkgClusterClient):
         self.generate_nodeconfig_file(ssl_file, nodeconfig_yaml_file)
         self.apply_yaml(nodeconfig_yaml_file)
 
+    def update_tkc(self, clusterName):
+        cmd = "tkc -n %s %s" % (clusterName, clusterName)
+        tkc = self.get_dict(cmd)
+        fqdn = self.repo.get_fqdn()
+
+        if 'airgap' in tkc['spec'] and tkc['spec']['airgap']['fqdn'] == fqdn:
+            logger.info("updating cluster [%s/%s] tkc spec airgap repo ca cert" % (clusterName, clusterName))
+            self.update_cluster_spec_ca(clusterName, clusterName)
+        
+        if ('annotations' in tkc['metadata'] and 
+              self.FQDN_ANNO in tkc['metadata']['annotations'] and
+              tkc['metadata']['annotations'][self.FQDN_ANNO] == fqdn):
+            logger.info("updating cluster [%s/%s] tkc airgap annotation repo ca cert" % (clusterName, clusterName))
+            self.update_cluster_anno_ca(clusterName, clusterName)
+
     def save_cluster_kubeconfig(self, path, clusterName, ns):
         kubeconfig_secret_name = "%s-kubeconfig" % clusterName
         kubeconfig_str = self.get_secret_data_value(ns, kubeconfig_secret_name)
