@@ -25,7 +25,7 @@ import pathlib
 import psycopg2
 from cluster import client
 
-CLI_VERSION = "v3.3.0"
+CLI_VERSION = "v3.3.0.1"
 
 TMP_DIR = "/tmp/update-ca"
 KUBE_CONFIG_FILE_PATTEN = "/tmp/update-ca/%s-kubeconfig.yaml"
@@ -192,7 +192,7 @@ def show_state_mgmtcluster(args):
         if mc_client.IsClusterClass(args.name):
             mc_client.cc_verify(args.name)
         else:
-            mc_client.verify()        
+            mc_client.verify(args.name)        
     else:
         client.logger.warning("the show_state_mgmtcluster command only run in in TCA-ControlPlane")
 
